$ErrorActionPreference = "Stop"

Set-ExecutionPolicy Bypass -Scope Process -Force

$env:chocolateyUseWindowsCompression = 'true'
$env:ChocolateyInstall = Convert-Path "$((Get-Command choco).path)\..\.."
Import-Module "$env:ChocolateyInstall\helpers\chocolateyProfile.psm1"

choco config set --name="'stopOnFirstPackageFailure'" --value="'true'"

function Write-PackageInstall($package) {
  Write-Output ""
  Write-Output "----------------------------------------------"
  Write-Output "Installing $package"
  Write-Output "----------------------------------------------"
  Write-Output ""
}
function Install-ChocoPackage($package, $options) {
  Write-PackageInstall $package
  choco install -y --no-progress $package $options
  if ($LastExitCode -ne 0) {
     throw 'Error installing with Chocolatey'
  }
}
function Ignore-SSLCertificates
{
    $Provider = New-Object Microsoft.CSharp.CSharpCodeProvider
    $Compiler = $Provider.CreateCompiler()
    $Params = New-Object System.CodeDom.Compiler.CompilerParameters
    $Params.GenerateExecutable = $false
    $Params.GenerateInMemory = $true
    $Params.IncludeDebugInformation = $false
    $Params.ReferencedAssemblies.Add("System.DLL") > $null
    $TASource=@'
        namespace Local.ToolkitExtensions.Net.CertificatePolicy
        {
            public class TrustAll : System.Net.ICertificatePolicy
            {
                public bool CheckValidationResult(System.Net.ServicePoint sp,System.Security.Cryptography.X509Certificates.X509Certificate cert, System.Net.WebRequest req, int problem)
                {
                    return true;
                }
            }
        }
'@ 
    $TAResults=$Provider.CompileAssemblyFromSource($Params,$TASource)
    $TAAssembly=$TAResults.CompiledAssembly
    ## We create an instance of TrustAll and attach it to the ServicePointManager
    $TrustAll = $TAAssembly.CreateInstance("Local.ToolkitExtensions.Net.CertificatePolicy.TrustAll")
    [System.Net.ServicePointManager]::CertificatePolicy = $TrustAll
}

$retryCount = 3
$retryDelay = 5


function Download-FileWithRetries {
    param (
        [string]$Url,
        [string]$OutFile
    )

    for ($retry = 1; $retry -le $retryCount; $retry++) {
        try {
            Write-Output "Attempt $($retry): Downloading $($Url)..."
            Invoke-WebRequest $Url -OutFile $OutFile -UseBasicParsing -UserAgent "curl/7.37.0" -ErrorAction Stop
            Write-Output "Download successful."
            return  
        } catch {
            Write-Output "Download failed. Retrying in $($retryDelay) seconds..."
            Start-Sleep -Seconds $($retryDelay)
        }
    }

    Write-Output "Maximum retry attempts reached. Download failed."
}
echo "======================= Installing mingw-get ======================"

# MinGW
$MINGWGET_DOWNLOAD_URL = "https://osdn.net/projects/mingw/downloads/68260/mingw-get-0.6.3-mingw32-pre-20170905-1-bin.zip"
$MINGWGET_DOWNLOAD_SHA = "2ab8efd7c7d1fc8eaf8b2fa4da4eef8f3e47768284c021599bc7435839a046df"

Write-Output "Downloading mingw-get"
Ignore-SSLCertificates
Download-FileWithRetries -Url $MINGWGET_DOWNLOAD_URL -OutFile mingw-get.zip
if ((Get-FileHash mingw-get.zip -Algorithm sha256).Hash -ne $MINGWGET_DOWNLOAD_SHA) {
  Write-Output 'CHECKSUM VERIFICATION FAILED!'
  exit 1
} else {
  Write-Output 'Verified checksum'
}

Expand-Archive mingw-get.zip -DestinationPath  C:\MinGW
Remove-Item -Force mingw-get.zip

# Update the system path
$oldSysPath = (Get-Itemproperty -path 'hklm:\system\currentcontrolset\control\session manager\environment' -Name Path).Path
$newSysPath = $oldSysPath + ";C:\MinGW\bin;"
Set-ItemProperty -path 'hklm:\system\currentcontrolset\control\session manager\environment' -Name Path -Value $newSysPath

# Update the current path
$env:Path += ";C:\MinGW\bin;"

echo "======================= Installing gcc 9.2.* ======================"
mingw-get install gcc="9.2.*" g++="9.2.*"
gcc --version


echo "=================== Install packages for cpp ======================"
mingw-get install mingw32-make

# CLANG
$LLVM_VERSION = "9.0.1"
Install-ChocoPackage  -package llvm -options --version=$LLVM_VERSION

refreshenv
clang --version

Write-Output "============= Installations complete ==========="
